

Imports System.Collections.Generic
Imports System.ComponentModel
Imports LC.Timers

' ------------------------------------------------------------------------ generates clock events internally
Public Class MidiInternalClock
	Inherits PpqnClock
    Implements IComponent

    ' -------------------------------------------------------------------- used for generating tick events
    Private timer As New Timer()
    ' -------------------------------------------------------------------- parses meta message tempo change messages
    Private builder As New TempoChangeBuilder()
    ' -------------------------------------------------------------------- tick accumulator
    Private m_ticks As Integer = 0
    ' -------------------------------------------------------------------- the clock has been disposed
    Private m_disposed As Boolean = False
    ' -------------------------------------------------------------------- 1 = normal time / 10 = fast play
    Private m_timewarping As Single = 1

    Private m_site As ISite = Nothing

    Public Sub New()
        MyBase.New(timer.Capabilities.periodMin)
        timer.Period = timer.Capabilities.periodMin
        AddHandler timer.Tick, New EventHandler(AddressOf HandleTick)
    End Sub

    Public Sub New(ByVal timerPeriod As Integer)
        MyBase.New(timerPeriod)
        timer.Period = timerPeriod
        AddHandler timer.Tick, New EventHandler(AddressOf HandleTick)
    End Sub

    Public Sub New(ByVal container As IContainer)
        MyBase.New(timer.Capabilities.periodMin)
        '
        ' Required for Windows.Forms Class Composition Designer support
        '
        container.Add(Me)
        '
        timer.Period = timer.Capabilities.periodMin
        AddHandler timer.Tick, New EventHandler(AddressOf HandleTick)
    End Sub

    Public Sub ClockStart()
        If m_disposed Then
            Throw New ObjectDisposedException("MidiInternalClock")
        End If
        If running Then
            Return
        End If
        m_ticks = 0
        Reset()
        OnStarted(EventArgs.Empty)
        ' -------------------------------------- start the multimedia timer in order to start generating ticks
        timer.TimerStart()
        running = True
    End Sub

    Public Sub ClockContinue()
        If m_disposed Then
            Throw New ObjectDisposedException("MidiInternalClock")
        End If
        If running Then
            Return
        End If
        ' -------------------------------------- raise Continued event
        OnContinued(EventArgs.Empty)
        ' -------------------------------------- start the multimedia timer in order to start generating ticks
        timer.TimerStart()
        running = True
    End Sub

    Public Sub ClockStop()
        If m_disposed Then
            Throw New ObjectDisposedException("MidiInternalClock")
        End If
        If Not running Then
            Return
        End If
        timer.TimerStop()
        running = False
        OnStopped(EventArgs.Empty)
    End Sub

    Public Sub SetTicks(ByVal ticks As Integer)
        If ticks < 0 Then
            Throw New ArgumentOutOfRangeException()
        End If
        If IsRunning Then
            ClockStop()
        End If
        Me.m_ticks = ticks
        Reset()
    End Sub

    Public Sub Process(ByVal message As MetaMessage)
        If message Is Nothing Then
            Throw New ArgumentNullException("message")
        End If
        If message.MetaType <> MetaType.Tempo Then
            Return
        End If
        Dim builder As New TempoChangeBuilder(message)
        Tempo = builder.Tempo
    End Sub

    Protected Overridable Sub OnDisposed(ByVal e As EventArgs)
        'Dim handler As EventHandler = Disposed
        'RaiseEvent handler(Me, e)
        RaiseEvent Disposed(Me, e)
    End Sub


    ' ------------------------------------------------------ handles Tick events generated by the multimedia timer
    '
    Private Sub HandleTick(ByVal sender As Object, ByVal e As EventArgs)
        Dim t As Integer = CInt(GenerateTicks() * m_timewarping)
        For i As Integer = 0 To t - 1
            OnTick(EventArgs.Empty)
            m_ticks += 1
        Next
    End Sub

    Public Property Tempo() As Integer
        Get
            If m_disposed Then
                Throw New ObjectDisposedException("MidiInternalClock")
            End If
            Return GetTempo()
        End Get
        Set(ByVal value As Integer)
            If m_disposed Then
                Throw New ObjectDisposedException("MidiInternalClock")
            End If
            SetTempo(value)
        End Set
    End Property

    Public Property TimeWarping() As Single
        Get
            Return m_timewarping
        End Get
        Set(ByVal value As Single)
            m_timewarping = value
        End Set
    End Property

    Public Overloads Overrides ReadOnly Property Ticks() As Integer
        Get
            Return m_ticks
        End Get
    End Property

    Public Event Disposed As EventHandler Implements System.ComponentModel.IComponent.Disposed

    Public Property Site() As ISite Implements IComponent.Site
        Get
            Return m_site
        End Get
        Set(ByVal value As ISite)
            m_site = value
        End Set
    End Property


    Public Sub Dispose() Implements IDisposable.Dispose
        If m_disposed Then
            Return
        End If
        If running Then
            timer.TimerStop()
        End If
        m_disposed = True
        timer.Dispose()
        GC.SuppressFinalize(Me)
        OnDisposed(EventArgs.Empty)
    End Sub

End Class
