


' --------------------------------------------------- converts a MIDI note number to its corresponding frequency
Public NotInheritable Class MidiNoteConverter

	Public Const NoteIDMinValue As Integer = 0
	Public Const NoteIDMaxValue As Integer = 127
	Private Shared ReadOnly NoteToFrequencyTable As Double() = New Double(NoteIDMaxValue) {}

    Shared Sub New()
        Dim notesPerOctave As Integer = 12
        Dim referenceFrequency As Double = 440
        Dim referenceNoteID As Integer = 69      ' note ID of the reference frequency
        Dim exponent As Double
        ' ------------------------------------------------------------- fill table with frequencies
        For i As Integer = 0 To NoteToFrequencyTable.Length - 1
            exponent = CDbl((i - referenceNoteID)) / notesPerOctave
            NoteToFrequencyTable(i) = referenceFrequency * Math.Pow(2, exponent)
        Next
    End Sub

	' Prevents instances of this class from being created - no need for
	' an instance to be created since this class only has static methods.
	Private Sub New()
	End Sub


    ' ------------------------------------------------------------- converts the specified note to a frequency
	'
	Public Shared Function NoteToFrequency(noteID As Integer) As Double
        If noteID < NoteIDMinValue OrElse noteID > NoteIDMaxValue Then
            Throw New ArgumentOutOfRangeException("Note ID out of range.")
        End If
        Return NoteToFrequencyTable(noteID)
    End Function

    ' ------------------------------------------------------------- converts the specified frequency to a note
    '
	Public Shared Function FrequencyToNote(frequency As Double) As Integer
		Dim noteID As Integer = 0
		Dim found As Boolean = False

        ' --------------------------------------------------------- search note with frequency near the specified
		Dim i As Integer = 0
		While i < NoteIDMaxValue AndAlso Not found
			noteID = i
            ' ----------------------------------------------------- frequency is less than next note frequency 
			If frequency < NoteToFrequency(noteID + 1) Then
                found = True
            End If
			i += 1
		End While

        ' --------------------------------------------------------- if the note is not first last note, narrow results
		If noteID > 0 AndAlso noteID < NoteIDMaxValue Then
            ' ----------------------------------------------------- get the previous note frequency
			Dim previousFrequncy As Double = NoteToFrequency(noteID - 1)
            ' ----------------------------------------------------- get the next note frequency
			Dim nextFrequency As Double = NoteToFrequency(noteID + 1)

            ' ------------------------------------------------------ next note is closer in frequency than previous note
			If nextFrequency - frequency < frequency - previousFrequncy Then
                ' -------------------------------------------------- move to the next note
				noteID += 1
			End If
		End If

		Return noteID
    End Function

End Class
