

Imports System.Collections.Generic

Public Class RecordingSession

    Private clock As IClock
    Private buffer As New List(Of TimestampedMessage)()
    Private m_result As New Track()

    Public Sub New(ByVal clock As IClock)
        Me.clock = clock
    End Sub

    Public Sub Build()
        m_result = New Track()
        buffer.Sort(New TimestampComparer())
        For Each tm As TimestampedMessage In buffer
            m_result.Insert(tm.ticks, tm.message)
        Next
    End Sub

    Public Sub Clear()
        buffer.Clear()
    End Sub

    Public ReadOnly Property Result() As Track
        Get
            Return m_result
        End Get
    End Property

    Public Sub Record(ByVal message As ChannelMessage)
        If clock.IsRunning Then
            buffer.Add(New TimestampedMessage(clock.Ticks, message))
        End If
    End Sub

    Public Sub Record(ByVal message As SysExMessage)
        If clock.IsRunning Then
            buffer.Add(New TimestampedMessage(clock.Ticks, message))
        End If
    End Sub

    Private Structure TimestampedMessage
        Public ticks As Integer
        Public message As IMidiMessage

        Public Sub New(ByVal ticks As Integer, ByVal message As IMidiMessage)
            Me.ticks = ticks
            Me.message = message
        End Sub
    End Structure

    Private Class TimestampComparer
        Implements IComparer(Of TimestampedMessage)

        Public Function Compare(ByVal x As TimestampedMessage, ByVal y As TimestampedMessage) As Integer Implements System.Collections.Generic.IComparer(Of TimestampedMessage).Compare
            If x.ticks > y.ticks Then
                Return 1
            ElseIf x.ticks < y.ticks Then
                Return -1
            Else
                Return 0
            End If
        End Function

    End Class

End Class
