
Imports System.Collections
Imports System.Collections.Generic
Imports System.IO

' ------------------------------------------------------------------------ Bank is a collection of Programs
Friend NotInheritable Class Bank
	Implements IEnumerable(Of Program)

    ' -------------------------------------------------------------------- use to identify Bank files
    Private Const BankFileHeader As String = "Synth Bank File"
    ' -------------------------------------------------------------------- the Programs belonging to the Bank
    Private programs As New List(Of Program)()
    ' -------------------------------------------------------------------- the name of the synthesizer the Bank belongs to
    Private synthName As String
    ' -------------------------------------------------------------------- the number of parameters in each Program
    Private parameterCount As Integer

    Public Sub New(ByVal synthName As String, ByVal parameterCount As Integer)
        If synthName Is Nothing Then
            Throw New ArgumentNullException("synthName")
        ElseIf parameterCount < 0 Then
            Throw New ArgumentOutOfRangeException("parameterCount")
        End If
        '
        Me.synthName = synthName
        Me.parameterCount = parameterCount
        ' ---------------------------------------------------------------- create Programs
        For i As Integer = 0 To 127
            programs.Add(New Program(synthName, parameterCount))
        Next
    End Sub

    ' -------------------------------------------------------------------- load Bank using the specified file name
    Public Sub Load(ByVal fileName As String)

        If Not My.Computer.FileSystem.FileExists(fileName) Then Exit Sub

        Using stream As New FileStream(fileName, FileMode.Open, FileAccess.Read)

            Dim reader As New BinaryReader(stream)
            ' ------------------------------------------------------------ if this isn't a bank file
            If reader.ReadString() <> BankFileHeader Then
                reader.Close()
                Throw New FileException("Wrong type of file.")
            End If
            ' ------------------------------------------------------------ if the synthesizer name does not match
            If reader.ReadString() <> synthName Then
                reader.Close()
                Throw New FileException("Wrong type of file.")
            End If
            '
            Dim p As Integer = reader.ReadInt32()
            ' ------------------------------------------------------------- choose the smaller number of parameters.
            '                                                               this gives some flexibility with changing
            '                                                               the number of parameters, though mismatching 
            '                                                               parameters to values is possible.
            p = (If(p < parameterCount, p, parameterCount))

            Dim newPrograms As New List(Of Program)()
            ' ------------------------------------------------------------- read programs from file
            For i As Integer = 0 To ProgramCount - 1
                newPrograms.Add(New Program(synthName, _
                                            reader.ReadString(), _
                                            parameterCount))
                ' --------------------------------------------------------- read parameter data into Program
                For n As Integer = 0 To p - 1
                    newPrograms(i)(n) = reader.ReadSingle()
                Next
            Next
            ' ------------------------------------------------------------- replace old Programs with new
            programs = newPrograms
            '
            reader.Close()
        End Using
    End Sub

    ' ---------------------------------------------------- Saves bank data to file using the specified file name.
    Public Sub Save(ByVal fileName As String)
        Using stream As New FileStream(fileName, FileMode.Create)
            Dim writer As New BinaryWriter(stream)
            '
            writer.Write(BankFileHeader)
            writer.Write(synthName)
            writer.Write(parameterCount)
            '
            For Each program As Program In programs
                writer.Write(program.Name)

                For i As Integer = 0 To parameterCount - 1
                    writer.Write(program(i))
                Next
            Next
            '
            writer.Close()
        End Using
    End Sub

    ' -------------------------------------------------------------------- get the Program at specified zero-based index
    Default Public ReadOnly Property Item(ByVal index As Integer) As Program
        Get
            If index < 0 OrElse index >= ProgramCount Then
                Throw New ArgumentOutOfRangeException("index")
            End If
            Return programs(index)
        End Get
    End Property

    ' -------------------------------------------------------------------- get the number of Programs
    Public ReadOnly Property ProgramCount() As Integer
        Get
            Return programs.Count
        End Get
    End Property

    Public Function GetEnumerator() As IEnumerator(Of Program) Implements IEnumerable(Of Program).GetEnumerator
        Return programs.GetEnumerator()
    End Function

    Private Function IEnumerable_GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
        Return programs.GetEnumerator()
    End Function

End Class
