

Imports System.Diagnostics
Imports LC.Wave

' ------------------------------------------------------------------------- stereo waveform data buffer

Public Class StereoBuffer
	Implements IProgrammable

    Public Enum ParameterId
        Amplitude
    End Enum

    ' ------------------------------------------------------ holding the stereo waveform data.
    Private buffer As Single(,)

    ' ------------------------------------------------------ for modulating the buffer's amplitude
    Private amplitude As Single = 0.25F

    ' ------------------------------------------------------ occurs when the value of the BufferSize property changes
    Public Event BufferSizeChanged As EventHandler


    Public Sub New(ByVal bufferSize As Integer)
        If bufferSize < 0 Then Throw New ArgumentOutOfRangeException("bufferSize")
        ' -------------------------------------------------- a two dimensional array, for left and right data
        buffer = New Single(1, bufferSize - 1) {}
    End Sub

    Public Sub Clear()
        Dim bufferSize__1 As Integer = BufferSize

        For i As Integer = 0 To bufferSize__1 - 1
            buffer(0, i) = InlineAssignHelper(buffer(1, i), 0)
        Next
    End Sub

    Public Function GetBuffer() As Single(,)
        Return buffer
    End Function

    ' ----------------------------------------------------- modulates the buffer amplitude by the amplitude value
    Friend Sub ModulateAmplitude()
        For i As Integer = 0 To BufferSize - 1
            buffer(0, i) *= amplitude
            buffer(1, i) *= amplitude
        Next
    End Sub

    ' -----------------------------------------------------  Gets the left channel peak.
    Friend Function GetLeftPeak() As Single
        Return GetPeak(0)
    End Function

    ' ----------------------------------------------------- Gets the right channel peak.
    Friend Function GetRightPeak() As Single
        Return GetPeak(1)
    End Function

    Private Function GetPeak(ByVal channel As Integer) As Single
        Dim result As Single = 0
        Dim sample As Single
        '
        For i As Integer = 0 To BufferSize - 1
            sample = Math.Abs(buffer(channel, i))
            If sample > result Then
                result = sample
            End If
        Next
        '
        Return result
    End Function

    ' ---------------------------------------------------------------- writes the buffer to a waveform OutputBuffer
    Friend Sub WriteToOutputBuffer(ByVal buffer As WaveformBuffer)
        Debug.Assert(buffer IsNot Nothing)
        Debug.Assert(buffer.Length >= BufferSize * Synthesizer.BlockAlign, "WaveformBuffer is too small.")

        Dim n As Integer = 0
        Dim v As Single
        Dim sample As Short

        For i As Integer = 0 To BufferSize - 1
          
            ' ---------------------------------------------------------- original
            'sample = (short)(short.MaxValue * this.buffer[0, i]);
            'buffer[n] = (byte)sample;
            'n++;
            'buffer[n] = (byte)(sample >> 8);
            'n++;
            ' ----------------------------------------------------------
            'sample = (short)(short.MaxValue * this.buffer[1, i]);
            'buffer[n] = (byte)sample;
            'n++;
            'buffer[n] = (byte)(sample >> 8);
            'n++;
            ' --------------------------------------------------------------------



            ' ---------------------------------------------------------- LEFT CHANNEL
            v = Me.buffer(0, i)
            ' ---------------------------------------------------------- hard limit. +1/-1
            If (v > 1.0F) Then v = 1.0F
            If (v < -1.0F) Then v = -1.0F
            'v = 0.5F * (Math.Abs(v + 1.0F) - Math.Abs(v - 1.0F))
            ' ---------------------------------------------------------- soft saturation
            'v = 1.5f * v - 0.5f * v * v * v;
            ' ---------------------------------------------------------- 
            sample = CShort(v * Short.MaxValue)
            ' ---------------------------------------------------------- 
            buffer(n) = CByte(sample And 255) 'CByte(sample)
            n += 1
            buffer(n) = CByte((sample >> 8) And 255) 'CByte((sample >> 8))
            n += 1
            ' ----------------------------------------------------------------------------


            ' ---------------------------------------------------------- RIGHT CHANNEL
            v = Me.buffer(1, i)
            ' ---------------------------------------------------------- hard limit. +1/-1
            If (v > 1.0F) Then v = 1.0F
            If (v < -1.0F) Then v = -1.0F
            'v = 0.5F * (Math.Abs(v + 1.0F) - Math.Abs(v - 1.0F))
            ' ---------------------------------------------------------- soft saturation
            'v = 1.5f * v - 0.5f * v * v * v
            ' ---------------------------------------------------------- 
            sample = CShort(v * Short.MaxValue)
            ' ---------------------------------------------------------- 
            buffer(n) = CByte(sample And 255) 'CByte(sample)
            n += 1
            buffer(n) = CByte((sample >> 8) And 255) 'CByte((sample >> 8))
            n += 1
            ' ----------------------------------------------------------------------------
        Next
    End Sub



    Protected Overridable Sub OnBufferSizeChanged(ByVal e As EventArgs)
        RaiseEvent BufferSizeChanged(Me, e)
    End Sub

    ' --------------------------------------------------------------------- Gets or sets the buffer size.
    Public Property BufferSize() As Integer
        Get
            Return buffer.GetLength(1)
        End Get
        Set(ByVal value As Integer)
            If value < 0 Then Throw New ArgumentOutOfRangeException("value")
            If value = buffer.GetLength(1) Then Return
            buffer = New Single(1, value - 1) {}
            OnBufferSizeChanged(EventArgs.Empty)
        End Set
    End Property


#Region "IProgrammable"

    Public Function GetParameterName(ByVal index As Integer) As String Implements IProgrammable.GetParameterName
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        '
        Dim result As String = String.Empty
        '
        Select Case CType(index, ParameterId)
            Case ParameterId.Amplitude
                result = "Amplitude"
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
        '
        Return result
    End Function

    Public Function GetParameterLabel(ByVal index As Integer) As String Implements IProgrammable.GetParameterLabel
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        '
        Dim result As String = String.Empty
        '
        Select Case CType(index, ParameterId)
            Case ParameterId.Amplitude
                result = "dB"
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
        '
        Return result
    End Function

    Public Function GetParameterDisplay(ByVal index As Integer) As String Implements IProgrammable.GetParameterDisplay
        '
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        '
        Dim result As String = String.Empty
        '
        Select Case CType(index, ParameterId)
            Case ParameterId.Amplitude
                result = Synthesizer.DecibelToString(amplitude)
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
        '
        Return result
    End Function

    Public Function GetParameterValue(ByVal index As Integer) As Single Implements IProgrammable.GetParameterValue
        '
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        '
        Dim result As Single = 0
        '
        Select Case CType(index, ParameterId)
            Case ParameterId.Amplitude
                result = amplitude
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
        '
        Return result
    End Function

    Public Sub SetParameterValue(ByVal index As Integer, ByVal value As Single) Implements IProgrammable.SetParameterValue
        '
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        ElseIf value < 0 OrElse value > 1 Then
            Throw New ArgumentOutOfRangeException("value")
        End If
        '
        Select Case CType(index, ParameterId)
            Case ParameterId.Amplitude
                amplitude = value
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
    End Sub

    Public ReadOnly Property ParameterCount() As Integer Implements IProgrammable.ParameterCount
        Get
            Return ParameterId.Amplitude + 1
        End Get
    End Property
    Private Shared Function InlineAssignHelper(Of T)(ByRef target As T, ByVal value As T) As T
        target = value
        Return value
    End Function

#End Region

End Class
