

Imports System.Collections.Generic
Imports System.Diagnostics
Imports LC.Midi

' -------------------------------------------------------- dispatching MIDI messages to the object responsible
'                                                           for handling specific types of MIDI messages
Friend Class MessageDispatcher


    Private allocator As VoiceAllocator
    Private bender As PitchBender
    Private processor As ControlProcessor

    Private m_omniEnabled As Boolean = False
    Private m_midiChannel As Integer = 0



    Public Sub New(ByVal allocator As VoiceAllocator, _
                   ByVal processor As ControlProcessor, _
                   ByVal bender As PitchBender)

        Debug.Assert(allocator IsNot Nothing)
        Debug.Assert(processor IsNot Nothing)
        Debug.Assert(bender IsNot Nothing)

        Me.allocator = allocator
        Me.bender = bender
        Me.processor = processor
    End Sub


    Public Sub Dispatch(ByVal message As ChannelMessage)
        Debug.Assert(message IsNot Nothing)

        CheckForOmniMessage(message)

        ' If the dispatcher should process this message.
        If m_omniEnabled OrElse message.MidiChannel = m_midiChannel Then
            If message.Command = ChannelCommand.NoteOn OrElse message.Command = ChannelCommand.NoteOff Then
                ' If this is a note-on or note-off message, dispatch it to the VoiceAllocator.

                allocator.Allocate(message)
            ElseIf message.Command = ChannelCommand.Controller Then
                ' Else if this is a controller message, dispatch it to the ControlProcessor.

                processor.Process(CType(message.Data1, ControllerType), message.Data2)
            ElseIf message.Command = ChannelCommand.PitchWheel Then
                ' Else if this is a pitch bend message, dispatch it to the PitchBender.

                bender.BendPitch(message.Data1, message.Data2)
            End If
        End If
    End Sub

    Private Sub CheckForOmniMessage(ByVal message As ChannelMessage)
        ' If this is a controller message.
        If message.Command = ChannelCommand.Controller Then
            Select Case CType(message.Data1, ControllerType)
                Case ControllerType.OmniModeOff
                    ' Message must be on the Synthesizer's "base" MIDI channel in order to be recognized.
                    If message.MidiChannel = m_midiChannel Then
                        OmniEnabled = False
                    End If

                Case ControllerType.OmniModeOn
                    ' Message must be on the Synthesizer's "base" MIDI channel in order to be recognized.
                    If message.MidiChannel = m_midiChannel Then
                        OmniEnabled = True
                    End If

            End Select
        End If
    End Sub


    ' -------------------------------------------------------------------- get or set the omni mode
    Public Property OmniEnabled() As Boolean
        Get
            Return m_omniEnabled
        End Get
        Set(ByVal value As Boolean)
            m_omniEnabled = value
        End Set
    End Property

    ' -------------------------------------------------------------------- get or set the MIDI channel
    Public Property MidiChannel() As Integer
        Get
            Return m_midiChannel
        End Get
        Set(ByVal value As Integer)
            If value < 0 OrElse value > ChannelMessage.MidiChannelMaxValue Then
                Throw New ArgumentOutOfRangeException("value")
            End If
            m_midiChannel = value
        End Set
    End Property

End Class
