

' ----------------------------------------------------------- functionality for mapping Waveforms to MIDI notes
Public Class WaveformMap

    ' The Waveforms from the Wavetable mapped to an array. 
    Private waveforms As Waveform() = New Waveform(SynthComponent.NoteCount - 1) {}

    Public Sub New(ByVal waves As Wavetable)
        If waves Is Nothing Then
            Throw New ArgumentNullException("waves")
        End If

        ' If there are any Waveforms in the Wavetable.
        If waves.Count > 0 Then
            ' Calculate the number of notes per Waveform.
            Dim notesPerWaveform As Integer = (If(waves.Count <= SynthComponent.NoteCount, SynthComponent.NoteCount \ waves.Count, 1))
            Dim noteIndex As Integer = 0
            Dim waveformIndex As Integer = 0

            ' While there are still waveforms to map to notes.
            While waveformIndex < waves.Count AndAlso noteIndex < SynthComponent.NoteCount
                ' Assign waveform to note range.
                For i As Integer = 0 To notesPerWaveform - 1
                    waveforms(noteIndex) = waves(waveformIndex)

                    noteIndex += 1
                Next

                waveformIndex += 1
            End While

            ' If there are still notes left.
            While noteIndex < waveforms.Length
                ' Assign the last waveform to the notes that are left.
                waveforms(noteIndex) = waves(waves.Count - 1)

                noteIndex += 1
            End While
        End If
    End Sub


    ' -------------------------------------------------- gets the Waveform that is mapped to the specified note
    '
    Default Public ReadOnly Property Item(ByVal note As Integer) As Waveform
        Get
            While note >= SynthComponent.NoteCount
                note -= SynthComponent.NotesPerOctave
            End While

            While note < 0
                note += SynthComponent.NotesPerOctave
            End While

            Return waveforms(note)
        End Get
    End Property

End Class
