Imports System.Windows.Forms

Public Class Class_MruList

    Private m_FileMenu As ToolStripMenuItem
    Private m_NumEntries As Integer
    Private m_FileNames As Collection
    Private m_MenuItems As Collection
    Friend Event OpenFile(ByVal file_name As String)

    Friend Sub New(ByVal file_menu As ToolStripMenuItem, ByVal num_entries As Integer)
        m_FileMenu = file_menu
        m_NumEntries = num_entries
        m_FileNames = New Collection
        m_MenuItems = New Collection
        ' ------------------------------------------------- Load saved file names
        LoadMruList()
        ' ------------------------------------------------- Display the MRU list
        DisplayMruList()
    End Sub

    Private Sub MruItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)
        Dim mnu As ToolStripMenuItem
        mnu = DirectCast(sender, ToolStripMenuItem)
        ' ------------------------------------------------- Find the menu item that raised this event
        For i As Integer = 1 To m_FileNames.Count
            ' --------------------------------------------- See if this is the item (add 1 for the separator)
            If m_MenuItems(i + 1) Is mnu Then
                ' ----------------------------------------- This is the item. Raise the OpenFile event for its file name
                RaiseEvent OpenFile(m_FileNames(i).ToString)
                Exit For
            End If
        Next i
    End Sub

    Private Sub DisplayMruList()
        ' -------------------------------------------------- Remove old menu items from the File menu
        For Each mnu As ToolStripItem In m_MenuItems
            m_FileMenu.DropDownItems.Remove(mnu)
        Next mnu
        m_MenuItems = New Collection
        ' -------------------------------------------------- See if we have any file names
        If m_FileNames.Count > 0 Then
            ' ---------------------------------------------- Make the separator
            Dim mnu As ToolStripItem = New ToolStripSeparator
            m_MenuItems.Add(mnu)
            m_FileMenu.DropDownItems.Add(mnu)
            ' ---------------------------------------------- Make the other menu items
            For i As Integer = 1 To m_FileNames.Count
                mnu = New ToolStripMenuItem( _
                    "&" & i & " " & IO.Path.GetFileName(m_FileNames(i).ToString), _
                    Nothing, _
                    New System.EventHandler(AddressOf MruItem_Click))
                m_MenuItems.Add(mnu)
                m_FileMenu.DropDownItems.Add(mnu)
            Next i
        End If
    End Sub

    Friend Sub Add(ByVal file_name As String)
        ' -------------------------------------------------- Remove this file from the MRU list if present.
        Dim i As Integer = FileNameIndex(file_name)
        If i > 0 Then m_FileNames.Remove(i)
        ' -------------------------------------------------- Add the item to the begining of the list.
        If m_FileNames.Count > 0 Then
            m_FileNames.Add(file_name, file_name, m_FileNames.Item(1))
        Else
            m_FileNames.Add(file_name, file_name)
        End If
        ' -------------------------------------------------- If the list is too long, remove the last item.
        If m_FileNames.Count > m_NumEntries Then
            m_FileNames.Remove(m_NumEntries + 1)
        End If
        ' -------------------------------------------------- Display the list.
        DisplayMruList()
        ' -------------------------------------------------- Save the updated list.
        SaveMruList()
    End Sub

    Private Function FileNameIndex(ByVal file_name As String) As Integer
        For i As Integer = 1 To m_FileNames.Count
            If m_FileNames(i).ToString = file_name Then Return i
        Next i
        Return 0
    End Function

    Friend Sub Remove(ByVal file_name As String)
        Dim i As Integer
        i = FileNameIndex(file_name)
        If i > 0 Then
            ' --------------------------------------------- Remove the file
            m_FileNames.Remove(i)
            ' --------------------------------------------- Display the list
            DisplayMruList()
            ' --------------------------------------------- Save the updated list
            SaveMruList()
        End If
    End Sub

    ' ==================================================================================================
    '  MRU List - Save Load
    ' ==================================================================================================
    Private Sub SaveMruList()
        Dim mruFileName As String = PlatformAdjustedFileName(Application.StartupPath & "\" & _
                                                             AssemblyName() & "_MruList.txt")
        Dim f As System.IO.StreamWriter = Nothing
        Try
            f = IO.File.CreateText(mruFileName)
            For i As Integer = 1 To m_FileNames.Count
                f.WriteLine(m_FileNames(i).ToString)
            Next
            f.Close()
        Catch
        End Try
        Try
            f.Close()
        Catch
        End Try
    End Sub

    Private Sub LoadMruList()
        ' ------------------------------------------------------------------------------- 
        ' With "Resume Next" subsequent parameters are loaded and f.Close() is executed
        ' -------------------------------------------------------------------------------
        On Error Resume Next  ' use Resume-Next instead of Try-Catch
        ' -------------------------------------------------------------------------------
        Dim l As String
        Dim mruFileName As String = PlatformAdjustedFileName(Application.StartupPath & "\" & _
                                                             AssemblyName() & "_MruList.txt")
        If My.Computer.FileSystem.FileExists(mruFileName) Then
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(mruFileName)
            Do While Not f.EndOfStream
                l = f.ReadLine()
                If l <> "" Then
                    m_FileNames.Add(l, l)
                End If
            Loop
            f.Close()
        End If
    End Sub

End Class
