﻿
Imports System.Diagnostics
Imports System.ComponentModel
Imports System.Windows.Forms
Imports LC.Multimedia
Imports LC.Midi
Imports LC.Midi.UI
Imports System.Drawing

Public Class Form1
    Inherits Form

    Private EventsAreEnabled As Boolean
    Private m_closing As Boolean = False
    Friend WithEvents m_MruList As Class_MruList
    Friend StartFileName As String = ""

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ' ---------------------------------------------------------------- 
        ToolTips_Init()
        Load_INI()
        ' ----------------------------------------------------------------
        ShowOutDeviceName()
        OpenOutputDevice()
        ' ---------------------------------------------------------------- 
        SetSyncModeParams()
        Text = AppTitleAndVersion("Theremino Midi Player")
        m_MruList = New Class_MruList(menu_File, 6)
        LoadFile(StartFileName)
        ' ---------------------------------------------------------------- 
        Me.Opacity = 100
        ' ---------------------------------------------------------------- 
        AddHandler sequence1.LoadProgressChanged, AddressOf HandleLoadProgressChanged
        AddHandler sequence1.LoadCompleted, AddressOf HandleLoadCompleted
        AddHandler sequencer1.PlayingCompleted, AddressOf Me.HandlePlayingCompleted
        AddHandler sequencer1.ChannelMessagePlayed, AddressOf Me.HandleChannelMessagePlayed
        AddHandler sequencer1.Stopped, AddressOf Me.HandleStopped
        AddHandler sequencer1.SysExMessagePlayed, AddressOf Me.HandleSysExMessagePlayed
        AddHandler sequencer1.Chased, AddressOf Me.HandleChased
        ' ----------------------------------------------------------------
        EventsAreEnabled = True
    End Sub

    Private Sub Form1_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        pianoControl1.PressPianoKey(e.KeyCode)
    End Sub

    Private Sub Form1_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyUp
        pianoControl1.ReleasePianoKey(e.KeyCode)
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        m_closing = True
    End Sub

    Private Sub Form1_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
        Save_INI()
        sequence1.Dispose()
        CloseOutputDevice()
    End Sub


    ' ===================================================================================
    '   MenuStrip and ToolStrip Gradients
    ' ===================================================================================
    Private Sub MenuStrip1_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles menuStrip1.Paint
        Dim bounds As New Rectangle(0, 0, _
                                    menuStrip1.Width, menuStrip1.Height)
        Dim brush As New Drawing2D.LinearGradientBrush(bounds, _
                                                       Color.FromArgb(230, 230, 230), _
                                                       Color.FromArgb(200, 200, 200), _
                                                       Drawing2D.LinearGradientMode.Vertical)
        e.Graphics.FillRectangle(brush, bounds)
    End Sub
    'Private Sub ToolStrip1_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles ToolStrip1.Paint
    '    Dim bounds As New Rectangle(0, 0, _
    '                                ToolStrip1.Width, ToolStrip1.Height)
    '    Dim brush As New Drawing2D.LinearGradientBrush(bounds, _
    '                                                   Color.White, _
    '                                                   Color.FromArgb(200, 200, 200), _
    '                                                   Drawing2D.LinearGradientMode.Vertical)
    '    e.Graphics.FillRectangle(brush, bounds)
    'End Sub

    ' ===================================================================================
    '   ToolStrip PressedButton color
    ' ===================================================================================
    'Class ToolStripButtonRenderer
    '    Inherits System.Windows.Forms.ToolStripProfessionalRenderer
    '    Protected Overrides Sub OnRenderButtonBackground(ByVal e As ToolStripItemRenderEventArgs)
    '        Dim btn As ToolStripButton = CType(e.Item, ToolStripButton)
    '        If btn IsNot Nothing AndAlso btn.CheckOnClick AndAlso btn.Checked Then
    '            Dim bounds As Rectangle = New Rectangle(0, 0, e.Item.Width - 1, e.Item.Height - 1)
    '            Dim brush As New Drawing2D.LinearGradientBrush(bounds, _
    '                                                           Color.Gold, _
    '                                                           Color.FromArgb(250, 250, 250), _
    '                                                           Drawing2D.LinearGradientMode.Vertical)
    '            e.Graphics.FillRectangle(brush, bounds)
    '            e.Graphics.DrawRectangle(Pens.Orange, bounds)
    '        Else
    '            MyBase.OnRenderButtonBackground(e)
    '        End If
    '    End Sub
    'End Class

    ' ===================================================================================
    '  MenuStrip and ToolStrip accepting the first click
    '  If the form receives a WM_PARENTNOTIFY (528) message and is not focused 
    '  then the form is activated before to exec the message
    ' ===================================================================================
    Protected Overrides Sub WndProc(ByRef m As Message)
        If m.Msg = 528 AndAlso Not Me.Focused Then
            Me.Activate()
        End If
        MyBase.WndProc(m)
    End Sub


    ' ==============================================================================================================
    '   ToolTips
    ' ==============================================================================================================
    Private ttp1 As ToolTip
    Private Sub ToolTips_Init()
        ttp1 = New ToolTip
        ttp1.ShowAlways = True
        ttp1.UseAnimation = True
        ttp1.UseFading = True
        ttp1.IsBalloon = True
        ttp1.AutoPopDelay = 32700
        ttp1.InitialDelay = 700
        ttp1.ReshowDelay = 500
        ' ------------------------------------------------------------- Add here Controls and ToolTip-Text
        ttp1.SetToolTip(txt_SingleChannel, _
                " Single channel" & vbCrLf & _
                " - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" & vbCrLf & _
                " The single MIDI channel to be played." & vbCrLf & _
                " The special vaue ""0"" means to play all channels.")
        ttp1.SetToolTip(chk_SyncMode, _
                " Sync mode" & vbCrLf & _
                " - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" & vbCrLf & _
                " The playing is synchronized from slot signals." & vbCrLf & _
                " Normally slot signals are generated with piezo sensors.")
        ttp1.SetToolTip(txt_SyncChannel, _
                " Sync channel" & vbCrLf & _
                " - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" & vbCrLf & _
                " The MIDI channel used for synchronization." & vbCrLf & _
                " The special vaue ""0"" synchronizes all channels.")
        ttp1.SetToolTip(txt_TimeWarping, _
                " Time warping" & vbCrLf & _
                " - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" & vbCrLf & _
                " Time distortion which allows you to play slow and fast at the same time." & vbCrLf & _
                " Too low values could prevent fast improvisations." & vbCrLf & _
                " Too high values may result in the loss of some notes." & vbCrLf & _
                " Normal values are 10 to 30.")
        ttp1.SetToolTip(txt_SyncSlot, _
                " Sync slot" & vbCrLf & _
                " - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" & vbCrLf & _
                " The first slot used for synchronization." & vbCrLf & _
                " The following slots are the last used notes." & vbCrLf & _
                " Normally 1 to 9 slots are used to play.")
        ttp1.SetToolTip(txt_SyncTrigLevel, _
                " Sync trig level" & vbCrLf & _
                " - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" & vbCrLf & _
                " The sync-slot level that triggers a Note-ON." & vbCrLf & _
                " Increase this value to reduce false triggerings." & vbCrLf & _
                " Normal trigger value is 100 to 200.")
        ttp1.SetToolTip(txt_VelocityMultiplier, _
                " Velocity multiplier" & vbCrLf & _
                " - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" & vbCrLf & _
                " This values increases or decreases the velocity of the notes." & vbCrLf & _
                " Normal value is 1.")
        ttp1.SetToolTip(txt_Compander, _
                " Compander" & vbCrLf & _
                " - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" & vbCrLf & _
                " This values expand or compress the velocity range." & vbCrLf & _
                " Normal value is 1.")
        ttp1.SetToolTip(chk_UseMidiVelocity, _
                " Use MIDI velocity" & vbCrLf & _
                " - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" & vbCrLf & _
                " OFF: The note velocity is controlled by the user only." & vbCrLf & _
                "  ON: The MIDI file velocity values are used too.")
        ttp1.SetToolTip(chk_SingleNoteOff, _
                " Single NoteOFF for all the MIDI channels" & vbCrLf & _
                " - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" & vbCrLf & _
                " Disabled: Each NoteON is followed with a corresponding NoteOFF." & vbCrLf & _
                " Enabled: A single NoteON is sent for all the channels." & vbCrLf & vbCrLf & _
                " Hint" & vbCrLf & _
                " - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" & vbCrLf & _
                " Disable to avoid notes that are playing for infinite time." & vbCrLf & _
                " Enable to avoid spurios NoteOFFs when channels are unified in the Synth.")
    End Sub

    ' ===================================================================================================
    '   TOOL STRIP MENU
    ' ===================================================================================================
    Private Sub openToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles openToolStripMenuItem.Click
        If StartFileName = "" Then
            openMidiFileDialog.InitialDirectory = Application.StartupPath & "\Media\Midi_Files"
            If Not FolderExists(openMidiFileDialog.InitialDirectory) Then
                openMidiFileDialog.InitialDirectory = Application.StartupPath & "\Midi\"
                If Not FolderExists(openMidiFileDialog.InitialDirectory) Then
                    openMidiFileDialog.InitialDirectory = Application.StartupPath & "\"
                End If
            End If
        End If
        If openMidiFileDialog.ShowDialog() = DialogResult.OK Then
            LoadFile(openMidiFileDialog.FileName)
        End If
    End Sub
    Private Sub m_MruList_OpenFile(ByVal filename As String) Handles m_MruList.OpenFile
        LoadFile(filename)
    End Sub
    Private Sub exitToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles exitToolStripMenuItem.Click
        Close()
    End Sub

    ' ===================================================================================================
    '   LOAD FILE
    ' ===================================================================================================
    Private Sub LoadFile(ByVal filename As String)
        ReleaseAllNotes()
        If filename Is Nothing Then Return
        filename = FindFileInAppFolders(filename)
        StartFileName = filename
        If filename = "" Then Return
        SyncHold = False
        sequencer1.Play_Stop()
        Me.Cursor = Cursors.WaitCursor
        startButton.Enabled = False
        stopButton.Enabled = False
        continueButton.Enabled = False
        positionHScrollBar.Enabled = False
        openToolStripMenuItem.Enabled = False
        sequence1.LoadAsync(filename)
        Text = AppTitleAndVersion("Theremino Midi Player") & " - " & IO.Path.GetFileName(filename)
        m_MruList.Add(filename)
        StartFileName = IO.Path.GetFileName(filename)
        Save_INI()
    End Sub
    Private Function FindFileInAppFolders(ByVal filename As String) As String
        'If My.Computer.FileSystem.FileExists(filename) Then
        '    If FileIsInAppFolders(filename) Then
        '        Return filename
        '    End If
        'End If
        If filename = "" Then Return ""
        Dim FileNames As Collections.ObjectModel.ReadOnlyCollection(Of String)
        Dim wildcards() As String = {"*.mid"}
        Dim s As String = Application.StartupPath & "\"
        FileNames = My.Computer.FileSystem.GetFiles(s, FileIO.SearchOption.SearchAllSubDirectories, wildcards)
        For Each str As String In FileNames
            If str.EndsWith(IO.Path.GetFileName(filename)) Then
                Return str
            End If
        Next
        MsgBox("File not found" & vbCrLf & filename)
        Return ""
    End Function
    'Private Function FileIsInAppFolders(ByVal filename As String) As Boolean
    '    If filename.StartsWith(Application.StartupPath) Then
    '        Return True
    '    Else
    '        Return False
    '    End If
    'End Function


    ' ====================================================================================================
    '   OUTPUT MIDI DEVICE
    ' ====================================================================================================
    Friend OutputDeviceID As Int32 = -1
    Private MidiOutDevice As OutputDevice
    Private Function OutputDeviceName(ByVal id As Int32) As String
        If id < 0 Or id >= OutputDevice.DeviceCount Then Return "Unused"
        Dim cap As MidiOutCaps = OutputDevice.GetDeviceCapabilities(id)
        Return cap.name
    End Function
    Private Sub ShowOutDeviceName()
        Combo_Init(cmb_OutputDevice, OutputDeviceName(OutputDeviceID))
    End Sub
    Private Sub OpenOutputDevice()
        CloseOutputDevice()
        If OutputDeviceID < 0 Or OutputDeviceID >= OutputDevice.DeviceCount Then Return
        MidiOutDevice = New OutputDevice(OutputDeviceID)
    End Sub
    Private Sub CloseOutputDevice()
        If MidiOutDevice Is Nothing Then Return
        MidiOutDevice.Close()
        MidiOutDevice = Nothing
    End Sub

    ' ====================================================================================================
    '   OUTPUT DEVICE COMBO
    ' ====================================================================================================
    Private Sub cmb_OutputDevice_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_OutputDevice.DropDown
        If OutputDevice.DeviceCount > 0 Then
            Dim oldname As String = cmb_OutputDevice.Items(0).ToString
            cmb_OutputDevice.ItemHeight = 15
            cmb_OutputDevice.Items.Clear()
            cmb_OutputDevice.Items.Add("Unused")
            For i As Integer = 0 To OutputDevice.DeviceCount - 1
                cmb_OutputDevice.Items.Add(OutputDeviceName(i))
            Next
            If OutputDeviceID >= 0 And OutputDeviceID < cmb_OutputDevice.Items.Count - 1 Then
                cmb_OutputDevice.SelectedIndex = OutputDeviceID + 1
            End If
            Combo_SetIndex_FromString(cmb_OutputDevice, oldname)
        End If
    End Sub
    Private Sub cmb_OutputDevice_DropDownClosed(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_OutputDevice.DropDownClosed
        cmb_OutputDevice.ItemHeight = 10
        Combo_Init(cmb_OutputDevice, cmb_OutputDevice.Text)
    End Sub
    Private Sub cmb_OutputDevice_SelectionChangeCommitted(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_OutputDevice.SelectionChangeCommitted
        OutputDeviceID = cmb_OutputDevice.SelectedIndex - 1
        OpenOutputDevice()
        Save_INI()
    End Sub


    ' ===================================================================================================
    '   USER COMMANDS
    ' ===================================================================================================
    Private Sub startButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles startButton.Click
        EventsAreEnabled = False
        ReleaseAllNotes()
        SyncHold = False
        sequencer1.Play_Start()
        timer1.Start()
        EventsAreEnabled = True
    End Sub
    Private Sub stopButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles stopButton.Click
        EventsAreEnabled = False
        ReleaseAllNotes()
        If Not chk_SyncMode.Checked Then
            sequencer1.Play_Stop()
            timer1.Stop()
        End If
        EventsAreEnabled = True
    End Sub
    Private Sub continueButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles continueButton.Click
        EventsAreEnabled = False
        ReleaseAllNotes()
        SyncHold = False
        sequencer1.Play_Continue()
        timer1.Start()
        EventsAreEnabled = True
    End Sub
    Private Sub positionHScrollBar_Scroll(ByVal sender As Object, ByVal e As ScrollEventArgs) Handles positionHScrollBar.Scroll
        If Not EventsAreEnabled Then Return
        EventsAreEnabled = False
        SyncHold = False
        sequencer1.Play_Hold()
        sequencer1.Position = e.NewValue
        timer1.Start()
        ShowInfo()
        sequencer1.Play_Continue()
        ReleaseKeyboardNotes()
        'ReleaseAllNotes()
        EventsAreEnabled = True
    End Sub
    Private Sub ReleaseKeyboardNotes()
        For i As Int32 = 0 To 127
            pianoControl1.ReleasePianoKey(i)
        Next
    End Sub
    Private Sub ReleaseAllNotes()
        ReleaseKeyboardNotes()
        If MidiOutDevice IsNot Nothing Then
            For ch As Int32 = 0 To 15
                MidiOutDevice.Send(New ChannelMessage(ChannelCommand.Controller, ch, ControllerType.AllNotesOff))
            Next
        End If
    End Sub

    ' ====================================================================================================
    '  SCROLLBAR TIMER and INFO
    ' ====================================================================================================
    Private Sub timer1_Tick(ByVal sender As Object, ByVal e As EventArgs) Handles timer1.Tick
        'If Not EventsAreEnabled Then Return
        '
        If sequencer1.Position <= sequence1.GetLength Then
            positionHScrollBar.Value = sequencer1.Position
        Else
            ' ---------------------------------- stop
            'SyncHold = False
            'sequencer1.Play_Stop()
            'timer1.Stop()
            ' ---------------------------------- auto repeat
            SyncHold = False
            sequencer1.Play_Start()
        End If
        '
        ShowInfo()
        '
        DecreaseSustain()
    End Sub
    Private Sub ShowInfo()
        Dim sec As Double
        sec = sequencer1.PositionSeconds
        Label1.Text = (Int(sec / 60)).ToString("00") & ":" & (sec Mod 60).ToString("00")
        sec = sequencer1.TotalSeconds
        Label2.Text = (Int(sec / 60)).ToString("00") & ":" & (sec Mod 60).ToString("00")
        Label3.Text = sequencer1.BPM().ToString("0 bpm")
    End Sub

    'Friend Function MouseButtonLeftPressed() As Boolean
    '    Return (Control.MouseButtons And Windows.Forms.MouseButtons.Left) <> Windows.Forms.MouseButtons.None
    'End Function


    ' ====================================================================================================
    '  PIANO KEYBOARD
    ' ====================================================================================================
    Private KeyboardChannel As Int32 = 0
    Private KeyboardVelocity As Int32 = 100
    Private Sub pianoControl1_PianoKeyDown(ByVal sender As Object, ByVal e As LC.Midi.UI.PianoKeyEventArgs) Handles pianoControl1.PianoKeyDown
        If Not EventsAreEnabled Then Return
        If MidiOutDevice Is Nothing Then Return
        Dim message As ChannelMessage = New ChannelMessage(ChannelCommand.NoteOn, KeyboardChannel, e.NoteID, KeyboardVelocity)
        MidiOutDevice.Send(message)
    End Sub
    Private Sub pianoControl1_PianoKeyUp(ByVal sender As Object, ByVal e As LC.Midi.UI.PianoKeyEventArgs) Handles pianoControl1.PianoKeyUp
        If Not EventsAreEnabled Then Return
        If MidiOutDevice Is Nothing Then Return
        Dim message As ChannelMessage = New ChannelMessage(ChannelCommand.NoteOff, KeyboardChannel, e.NoteID)
        MidiOutDevice.Send(message)
    End Sub
    Private Sub PianoControl_UpdatePressedKeys(ByVal message As ChannelMessage)
        EventsAreEnabled = False
        If message.Command = ChannelCommand.NoteOn AndAlso message.Data2 > 0 Then
            pianoControl1.PressPianoKey(message.Data1)
        Else
            pianoControl1.ReleasePianoKey(message.Data1)
        End If
        EventsAreEnabled = True
    End Sub


    ' ===================================================================================================
    '   sequencer1 - load handlers
    ' ===================================================================================================
    Private Sub HandleLoadProgressChanged(ByVal sender As Object, ByVal e As ProgressChangedEventArgs)
        'toolStripProgressBar1.Value = e.ProgressPercentage
    End Sub

    Private Sub HandleLoadCompleted(ByVal sender As Object, ByVal e As AsyncCompletedEventArgs)
        Me.Cursor = Cursors.Arrow
        startButton.Enabled = True
        continueButton.Enabled = True
        stopButton.Enabled = True
        positionHScrollBar.Enabled = True
        openToolStripMenuItem.Enabled = True
        'toolStripProgressBar1.Value = 0
        If e.[Error] Is Nothing Then
            positionHScrollBar.Maximum = sequence1.GetLength()
            positionHScrollBar.LargeChange = sequence1.GetLength \ 20
            positionHScrollBar.Value = 0
            sequencer1.Position = 0
            ShowInfo()
            '
            SyncHold = False
            sequencer1.Play_Start()
            timer1.Start()
        Else
            MessageBox.Show(e.[Error].Message)
        End If
    End Sub


    ' ===================================================================================================
    '   sequencer1 - handlers
    ' ===================================================================================================
    Private Sub HandleChased(ByVal sender As Object, ByVal e As ChasedEventArgs)
        If MidiOutDevice Is Nothing Then Return
        For Each message As ChannelMessage In e.Messages
            MidiOutDevice.Send(message)
        Next
    End Sub
    Private Sub HandleSysExMessagePlayed(ByVal sender As Object, ByVal e As SysExMessageEventArgs)
        'sometimes causes an exception because the output device is overloaded
        If MidiOutDevice IsNot Nothing Then
            MidiOutDevice.Send(e.Message)
        End If
    End Sub
    Private Sub HandleStopped(ByVal sender As Object, ByVal e As StoppedEventArgs)
        If MidiOutDevice Is Nothing Then Return
        For Each message As ChannelMessage In e.Messages
            PianoControl_UpdatePressedKeys(message)
            MidiOutDevice.Send(message)
        Next
    End Sub
    Private Sub HandlePlayingCompleted(ByVal sender As Object, ByVal e As EventArgs)

    End Sub



    ' ===================================================================================================
    '   sequencer1 - channel message handler
    ' ===================================================================================================
    Private vol As Single = 0

    Private Sub HandleChannelMessagePlayed(ByVal sender As Object, ByVal e As ChannelMessageEventArgs)
        If m_closing Then Return

        If Not EventsAreEnabled Then Return

        Dim MsgCommand As ChannelCommand = e.Message.Command
        Dim MsgChannel As Int32 = e.Message.MidiChannel
        Dim MsgNote As Int32 = e.Message.Data1
        Dim MsgVelocity As Int32 = e.Message.Data2

        ' ------------------------------------------------------------------------- sync mode
        If SyncSlot >= 0 Then
            If SyncChannel = -1 OrElse SyncChannel = MsgChannel Then
                If MsgCommand = ChannelCommand.NoteOn Then
                    If MsgVelocity > 0 Then
                        sequencer1.Play_Hold()
                        SyncHold = True
                        Do
                            'Static sw As Stopwatch = New Stopwatch
                            'sw.Reset()
                            'sw.Start()
                            System.Threading.Thread.Sleep(1)
                            'If sw.ElapsedMilliseconds > 6 Then
                            '    Debug.Print(sw.ElapsedMilliseconds.ToString)
                            'End If

                            If Not SyncHold Then Return
                            If SyncSlot < 0 Then Exit Do
                            'If My.Computer.Keyboard.CtrlKeyDown Then Exit Do
                            vol = PollSlots()
                            If vol > 0 Then
                                Exit Do
                            End If

                        Loop
                        sequencer1.Play_HoldRestart()
                    End If
                End If
            End If
            ' ------------------------------------------------------------------------- 
            If SingleChannel = -1 OrElse SingleChannel = MsgChannel Then
                PianoControl_UpdatePressedKeys(e.Message)
                If MidiOutDevice IsNot Nothing Then
                    If MsgCommand = ChannelCommand.NoteOn Then
                        If e.Message.Data2 > 0 Then
                            ' --------------------------------------------------------- prepare a new velocity
                            Dim velocity As Int32
                            If vol < 0 Then vol = 0
                            If UseMidiVelocity Then
                                velocity = CInt(vol * 1.3F * e.Message.Data2)
                            Else
                                velocity = CInt(vol * 120)
                            End If
                            If velocity > 127 Then velocity = 127

                            SendNoteOnToDevice(MsgChannel, MsgNote, velocity)
                            PianoControl_UpdatePressedKeys(e.Message)
                            ' --------------------------------------------------------- put note in the FIFO
                            LastMsgIndex += 1
                            If LastMsgIndex > 60 Then LastMsgIndex = 0
                            LastMsg(LastMsgIndex) = e.Message
                        End If
                    End If
                End If
            End If
        Else
            ' ------------------------------------------------------------------------- normal playing
            If SingleChannel = -1 OrElse SingleChannel = MsgChannel Then
                PianoControl_UpdatePressedKeys(e.Message)
                If MidiOutDevice IsNot Nothing Then
                    MidiOutDevice.Send(e.Message)
                End If
            End If
        End If

    End Sub


    ' ===================================================================================================
    '   SYNC MODE
    ' ===================================================================================================
    Private SyncHold As Boolean
    Private SyncBpm As Int32 = 100
    Private SyncSlot As Int32 = -1
    Private SyncTrigLevel As Int32 = 100
    Private SyncChannel As Int32 = -1
    Private SingleChannel As Int32 = -1

    Private Sub SetSyncModeParams()
        If chk_SyncMode.Checked Then
            Me.Width = txt_TimeWarping.Right + 12
            SyncBpm = txt_TimeWarping.NumericValueInteger
            SyncSlot = txt_SyncSlot.NumericValueInteger
            SyncTrigLevel = txt_SyncTrigLevel.NumericValueInteger
            SyncChannel = txt_SyncChannel.NumericValueInteger - 1
            VelocityMultiplier = CSng(txt_VelocityMultiplier.NumericValue)
            Compander = CSng(txt_Compander.NumericValue)
            UseMidiVelocity = chk_UseMidiVelocity.Checked()
            SingleNoteOff = chk_SingleNoteOff.Checked()
            sequencer1.TimeWarping = txt_TimeWarping.NumericValueInteger
            If Not timer1.Enabled Then
                sequencer1.Play_Continue()
                timer1.Start()
            End If
        Else
            Me.Width = positionHScrollBar.Right + 14
            SyncSlot = -1
            SyncChannel = -1
        End If

        SingleChannel = txt_SingleChannel.NumericValueInteger - 1
    End Sub
    Private Sub SyncMode_LostFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) _
                                                                Handles txt_SingleChannel.LostFocus, _
                                                                        chk_SyncMode.LostFocus, _
                                                                        txt_SyncChannel.LostFocus, _
                                                                        txt_TimeWarping.LostFocus, _
                                                                        txt_SyncSlot.LostFocus, _
                                                                        txt_SyncTrigLevel.LostFocus, _
                                                                        txt_SingleChannel.LostFocus, _
                                                                        txt_VelocityMultiplier.LostFocus, _
                                                                        txt_Compander.LostFocus, _
                                                                        chk_UseMidiVelocity.LostFocus, _
                                                                        chk_SingleNoteOff.LostFocus
        If Not EventsAreEnabled Then Return
        Save_INI()
    End Sub
    Private Sub SyncMode_ParamChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
                                                                        Handles txt_SingleChannel.TextChanged, _
                                                                                chk_SyncMode.CheckedChanged, _
                                                                                txt_SyncChannel.TextChanged, _
                                                                                txt_TimeWarping.TextChanged, _
                                                                                txt_SyncSlot.TextChanged, _
                                                                                txt_SyncTrigLevel.TextChanged, _
                                                                                txt_VelocityMultiplier.TextChanged, _
                                                                                txt_Compander.TextChanged, _
                                                                                chk_UseMidiVelocity.CheckedChanged, _
                                                                                chk_SingleNoteOff.CheckedChanged
        If Not EventsAreEnabled Then Return
        SetSyncModeParams()
    End Sub


    Private Sub chk_SyncMode_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
                                                                            Handles chk_SyncMode.CheckedChanged
        If Not EventsAreEnabled Then Return
        If chk_SyncMode.Checked Then
            sequencer1.TimeWarping = txt_TimeWarping.NumericValueInteger
            PreselectAllSustainedNotesForNoteOFF()
        Else
            sequencer1.TimeWarping = 1
        End If
        ReleaseAllNotes()
    End Sub


    ' ===================================================================================================
    '  POLL SLOTS
    ' ===================================================================================================
    Private LastMsg(64) As LC.Midi.ChannelMessage
    Private LastMsgIndex As Int32 = 0

    Private OldValues(128) As Single
    Private Ready(128) As Int32
    Private NumSlots As Int32 = 9

    Private Compander As Single = 1
    Private VelocityMultiplier As Single = 1
    Private UseMidiVelocity As Boolean = False
    Private SingleNoteOff As Boolean = False

    Private Function PollSlots() As Single
        If MidiOutDevice Is Nothing Then Return 0
        ' ---------------------------------------------------------------------- increased dynamics
        Dim kgain As Single = VelocityMultiplier / (1000 - SyncTrigLevel)
        Dim v As Single
        Dim vol As Single
        PollSlots = -1
        For n As Int32 = 0 To NumSlots - 1
            v = Slots.ReadSlot_NoNan(SyncSlot + n)
            If Ready(n) >= 1 Then
                If v < OldValues(n) * 0.9 Then

                    vol = (OldValues(n) - SyncTrigLevel) * kgain
                    If vol < 0 Then vol = 0
                    vol = CSng(vol ^ Compander)

                    If n = 0 Then
                        PollSlots = vol
                        'Debug.Print(CInt(vol * 128).ToString)
                    Else
                        Dim i As Int32 = LastMsgIndex + 1 - n
                        If i < 0 Then i += 60
                        If LastMsg(i) IsNot Nothing Then
                            With LastMsg(i)
                                Dim velocity As Int32
                                If UseMidiVelocity Then
                                    velocity = CInt(vol * 1.3F * LastMsg(i).Data2)
                                Else
                                    velocity = CInt(vol * 120)
                                End If
                                If velocity > 127 Then velocity = 127
                                SendNoteOnToDevice(.MidiChannel, .Data1, velocity)
                                PianoControl_UpdatePressedKeys(LastMsg(i))
                            End With
                        End If
                    End If
                    ' --------------------------------------------------------- no fast note - wait less than trigger
                    Ready(n) = -1
                End If
            ElseIf Ready(n) = -1 Then
                ' ------------------------------------------------------------- wait value less than trigger 
                If v <= SyncTrigLevel Then
                    Ready(n) = 0
                End If
            Else
                ' ------------------------------------------------------------- wait a rising front
                If v > OldValues(n) Then
                    ' --------------------------------------------------------- and value more than TriggerValue
                    If v > SyncTrigLevel Then
                        Ready(n) = 1
                    End If
                End If
            End If
            OldValues(n) = v
        Next

    End Function


    ' ===================================================================================================
    '  SUSTAIN NOTES - NoteOFF are posposed by a little sustain time
    ' ===================================================================================================
    Private SustainDecreaseSpeed As Int32 = 5
    Private SustainList As List(Of LC.Midi.ChannelMessage) = New List(Of LC.Midi.ChannelMessage)
    Private Sub SendNoteOnToDevice(ByVal channel As Int32, ByVal note As Int32, ByVal velocity As Int32)
        If velocity < 0 Then velocity = 0
        If velocity > 127 Then velocity = 127
        If CanChangeNote(channel, note, velocity) Then
            Dim m As LC.Midi.ChannelMessage
            m = New LC.Midi.ChannelMessage(ChannelCommand.NoteOn, channel, note, velocity)
            SustainList.Add(m)
            MidiOutDevice.Send(m)
        End If
    End Sub
    Private Sub DecreaseSustain()
        If MidiOutDevice Is Nothing Then Return
        For i As Int32 = SustainList.Count - 1 To 0 Step -1
            If SustainList(i).Data2 > SustainDecreaseSpeed Then
                SustainList(i).Data2 -= SustainDecreaseSpeed
                If SustainList(i).Data2 <= SustainDecreaseSpeed Then
                    If CanChangeNote(SustainList(i).MidiChannel, SustainList(i).Data1, SustainDecreaseSpeed + 1) Then
                        SustainList(i).Command = ChannelCommand.NoteOff
                        SustainList(i).Data2 = 0
                        MidiOutDevice.Send(SustainList(i))
                        PianoControl_UpdatePressedKeys(SustainList(i))
                    End If
                    SustainList.RemoveAt(i)
                End If
            End If
        Next
    End Sub
    Private Sub PreselectAllSustainedNotesForNoteOFF()
        For i As Int32 = 0 To 127
            For ch As Int32 = 0 To 15
                SustainList.Add(New LC.Midi.ChannelMessage(ChannelCommand.NoteOn, ch, i, 10))
            Next
        Next
    End Sub
    ' -------------------------------------------------------------------------------------------------------
    '  If the same note is playing with "sustained" velocity, than the note can not be changed
    ' -------------------------------------------------------------------------------------------------------
    Private Function CanChangeNote(ByVal channel As Int32, ByVal note As Int32, ByVal velocity As Int32) As Boolean
        Try
            For Each m As LC.Midi.ChannelMessage In SustainList
                If m.Data1 = note Then
                    If m.Data2 >= velocity Then
                        If m.MidiChannel = channel Or SingleNoteOff Then
                            Return False
                        End If
                    End If
                End If
            Next
        Catch
        End Try
        Return True
    End Function

End Class
