﻿
Imports System.Drawing.Imaging

Module Module_SaveLoad

    Friend EventsAreEnabled As Boolean = False

    ' =======================================================================================================
    '   APP TITLE AND VERSION
    ' =======================================================================================================
    Friend Function AppTitleAndVersion(Optional ByVal Title As String = "") As String
        If Title = "" Then Title = Replace(My.Application.Info.AssemblyName, "_", " ")
        Dim s() As String = Split(My.Application.Info.Version.ToString, ".")
        Return Title & " - V" & s(0) & "." & s(1)
    End Function

    ' =======================================================================================================
    '   Files
    ' =======================================================================================================
    ' returns lower-case extension with initial dot
    Public Function GetExtension(ByVal str As String) As String
        On Error Resume Next
        Return LCase(IO.Path.GetExtension(str))
    End Function
    Public Function RemoveExtension(ByVal str As String) As String
        On Error Resume Next
        Return PlatformAdjustedFileName(IO.Path.GetDirectoryName(str) & "\" & IO.Path.GetFileNameWithoutExtension(str))
    End Function
    Public Function FileExists(ByVal fileName As String) As Boolean
        Return My.Computer.FileSystem.FileExists(fileName)
    End Function
    Public Function FolderExists(ByVal FolderName As String) As Boolean
        If FolderName.Length < 2 Then Return False
        FolderName = LCase(FolderName)
        Select Case FolderName
            Case "a:\", "b:\", "c:\", "d:\", "e:\", "f:\", "g:\", "h:\", "i:\", "j:\", "k:\"
                Return True
        End Select
        Return My.Computer.FileSystem.DirectoryExists(FolderName)
    End Function

    ' =======================================================================================
    '  FORM FUNCTIONS
    ' =======================================================================================
    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
        'EnsureFormCompletelyVisible(f)
    End Sub

    Private SB As Rectangle = New Rectangle(Integer.MaxValue, Integer.MaxValue, Integer.MinValue, Integer.MinValue)

    Private Sub GetMaxScreenBounds()
        For Each s As Screen In System.Windows.Forms.Screen.AllScreens
            SB = Rectangle.Union(SB, s.WorkingArea)
        Next
    End Sub

    'Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
    '    With frm
    '        .Width = Math.Min(.Width, SB.Width)         ' not more than a maximized window
    '        .Height = Math.Min(.Height, SB.Height)      ' not more than a maximized window
    '        .Width = Math.Max(.Width, 32)               ' at least 32x24
    '        .Height = Math.Max(.Height, 24)             ' at least 32x24
    '        .Left = Math.Min(.Left, SB.Right - .Width)  ' not beyond the right border
    '        .Top = Math.Min(.Top, SB.Bottom - .Height)  ' not beyond the bottom border
    '        .Left = Math.Max(.Left, SB.Left)            ' at least at the left border
    '        .Top = Math.Max(.Top, SB.Top)               ' at least at the top border
    '    End With
    'End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            If .Left < -9000 Then .Left = 0
            .Width = Math.Min(.Width, SB.Width)             ' not more than VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, SB.Height)          ' not more than VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)                   ' at least 32x24
            .Height = Math.Max(.Height, 24)                 ' at least 32x24
            .Left = Math.Min(.Left, SB.Right - 50)          ' not beyond right border - 50 pixels
            .Top = Math.Min(.Top, SB.Bottom - 100)          ' not beyond bottom border - 50 pixels
            .Left = Math.Max(.Left, SB.Left + 100 - .Width) ' at least at left border + 50 pixels
            .Top = Math.Max(.Top, SB.Top - 10)              ' at least at top border
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function


    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================
    Private Function TabString(ByVal Name As String, _
                              Optional ByVal Value As Double = Double.NaN, _
                              Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt)
        End If
    End Function
    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function
    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function
    Private Function Val_Single(ByVal l As String) As Single
        Return CSng(Val(l.Replace(",", ".")))
    End Function
    Private Function Val_Double(ByVal l As String) As Double
        Return Val(l.Replace(",", "."))
    End Function
    Private Function Val_Int(ByVal l As String) As Int32
        Return CInt(Val(l))
    End Function
    Private Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Integer
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function
    Friend Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function


    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================
    Friend Sub LoadConfiguration()
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.InitialDirectory = PlatformAdjustedFileName(Application.StartupPath & "\media\Configurations")
        ofd.Filter = "Configuration file (*.txt)|*txt"
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Load_INI(ofd.FileName)
            Form1.ConfigFileName = IO.Path.GetFileNameWithoutExtension(ofd.FileName)
        End If
    End Sub

    Friend Sub SaveConfigurationAs()
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.InitialDirectory = PlatformAdjustedFileName(Application.StartupPath & "\media\Configurations")
        sfd.DefaultExt = ".txt"
        sfd.Filter = "Configuration file (*.txt)|*txt"
        sfd.FileName = Form1.ConfigFileName
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Save_INI(sfd.FileName)
        End If
    End Sub

    Friend Sub Save_INI(Optional ByVal iniFileName As String = "")
        '
        If iniFileName = "" Then
            iniFileName = PlatformAdjustedFileName(Application.StartupPath & "\" & AssemblyName() & "_INI.txt")
        End If
        iniFileName = PlatformAdjustedFileName(iniFileName)
        '
        Dim f As System.IO.StreamWriter = Nothing
        Try
            f = IO.File.CreateText(iniFileName)
            '
            f.WriteLine(" Program Params")
            f.WriteLine("===========================================")
            ' ------------------------------------------------------------------------------ FORM BOUNDS
            Dim r As Rectangle
            r = GetFormRectangle(Form1)
            f.WriteLine(TabString("Form1_Top", r.Top))
            f.WriteLine(TabString("Form1_Left", r.Left))
            'f.WriteLine(TabString("Form1_Width", r.Width))
            'f.WriteLine(TabString("Form1_Height", r.Height))
            f.WriteLine(TabString("Form1_WindowState", Form1.WindowState))
            '
            f.WriteLine("")
            f.WriteLine(" Params")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("NoteSlot", Form1.txt_NoteSlot.NumericValueInteger))
            f.WriteLine(TabString("TriggerLevel", Form1.txt_TriggerLevel.NumericValueInteger))
            f.WriteLine(TabString("VolumeSlot", Form1.txt_VolumeSlot.NumericValueInteger))
            f.WriteLine(TabString("VolumeNegative", Form1.chk_VolumeNegative.Checked))
            ' ----------------------------------------------------------------------------------
            f.WriteLine(TabString("PlayEffects", Form1.cmb_PlayEffects.SelectedIndex))
            f.WriteLine(TabString("Amount", Form1.txt_Amount.NumericValueInteger))
            ' ----------------------------------------------------------------------------------
            f.WriteLine(TabString("PlayerType", Form1.cmb_PlayerType.SelectedIndex))
            f.WriteLine(TabString("PollingTime", Form1.txt_PollingTime.NumericValueInteger))
            f.WriteLine(TabString("BufferLen", Form1.txt_BufferLen.NumericValueInteger))
            ' ----------------------------------------------------------------------------------
            f.WriteLine(TabString("VoiceSlotPlus", Form1.txt_VoiceSlotPlus.NumericValueInteger))
            f.WriteLine(TabString("VoiceSlotMinus", Form1.txt_VoiceSlotMinus.NumericValueInteger))
            f.WriteLine(TabString("ChordSlotPlus", Form1.txt_ChordSlotPlus.NumericValueInteger))
            f.WriteLine(TabString("ChordSlotMinus", Form1.txt_ChordSlotMinus.NumericValueInteger))
            f.WriteLine(TabString("ChordTime", Form1.txt_ChordTime.NumericValueInteger))
            ' ----------------------------------------------------------------------------------
            f.WriteLine(TabString("OutputDeviceID", Form1.OutputDeviceID))
            f.WriteLine(TabString("stm_MidiChannel", Form1.txt_OutMidiChannel.NumericValueInteger))
            ' ----------------------------------------------------------------------------------
            f.WriteLine(TabString("ConfigFileName", Form1.ConfigFileName))
            f.WriteLine(TabString("SelectedVoice", Form1.synth1.SelectedProgramIndex))
            f.WriteLine(TabString("ChordFileName", Form1.ChordFileName))
            f.WriteLine(TabString("SelectedTab", Form1.tabControl1.SelectedIndex))
        Catch
        End Try
        Try
            f.Close()
        Catch
        End Try
    End Sub


    Friend Sub Load_INI(Optional ByVal iniFileName As String = "")
        ' ------------------------------------------------------------------------------- defaults
        '
        ' -------------------------------------------------------------------------------
        ' ------------------------------------------------------------------------------- 
        ' With "Resume Next" subsequent parameters are loaded and f.Close() is executed
        ' -------------------------------------------------------------------------------
        On Error Resume Next  ' use Resume-Next instead of Try-Catch
        ' -------------------------------------------------------------------------------
        Dim LoadAll As Boolean = False
        If iniFileName = "" Then
            iniFileName = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
            LoadAll = True
            ' ----------------------------------------------------------- default values
            Form1.Top = 0
            Form1.Left = 0
        End If
        iniFileName = PlatformAdjustedFileName(iniFileName)

        Dim l As String
        If My.Computer.FileSystem.FileExists(iniFileName) Then
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)

            Do While Not f.EndOfStream
                l = f.ReadLine()
                Select Case ExtractParamName(l)
                    Case "Form1_Top" : If LoadAll Then Form1.Top = Val_Int(l)
                    Case "Form1_Left" : If LoadAll Then Form1.Left = Val_Int(l)
                        'Case "Form1_Width" : If LoadAll Then Form1.Width = Val_Int(l)
                        'Case "Form1_Height" : If LoadAll Then Form1.Height = Val_Int(l)
                    Case "Form1_WindowState" : If LoadAll Then Form1.WindowState = CType((Val(l)), FormWindowState)
                        ' --------------------------------------------------------------------------
                    Case "NoteSlot" : Form1.txt_NoteSlot.NumericValueInteger = Val_Int(l)
                    Case "TriggerLevel" : Form1.txt_TriggerLevel.NumericValueInteger = Val_Int(l)
                    Case "VolumeSlot" : Form1.txt_VolumeSlot.NumericValueInteger = Val_Int(l)
                    Case "VolumeNegative" : Form1.chk_VolumeNegative.Checked = l = "True"
                        ' --------------------------------------------------------------------------
                    Case "PlayEffects" : Form1.cmb_PlayEffects.SelectedIndex = CInt(Val(l))
                    Case "Amount" : Form1.txt_Amount.NumericValue = Val_Int(l)
                        ' --------------------------------------------------------------------------
                    Case "PlayerType" : Form1.cmb_PlayerType.SelectedIndex = CInt(Val(l))
                    Case "PollingTime" : Form1.txt_PollingTime.NumericValueInteger = Val_Int(l)
                    Case "BufferLen" : Form1.txt_BufferLen.NumericValueInteger = Val_Int(l)
                        ' --------------------------------------------------------------------------
                    Case "VoiceSlotPlus" : Form1.txt_VoiceSlotPlus.NumericValueInteger = Val_Int(l)
                    Case "VoiceSlotMinus" : Form1.txt_VoiceSlotMinus.NumericValueInteger = Val_Int(l)
                    Case "ChordSlotPlus" : Form1.txt_ChordSlotPlus.NumericValueInteger = Val_Int(l)
                    Case "ChordSlotMinus" : Form1.txt_ChordSlotMinus.NumericValueInteger = Val_Int(l)
                    Case "ChordTime" : Form1.txt_ChordTime.NumericValueInteger = Val_Int(l)
                        ' --------------------------------------------------------------------------
                    Case "OutputDeviceID" : Form1.OutputDeviceID = Val_Int(l)
                    Case "stm_MidiChannel" : Form1.txt_OutMidiChannel.NumericValueInteger = Val_Int(l)
                        ' --------------------------------------------------------------------------
                    Case "ConfigFileName" : Form1.ConfigFileName = l
                    Case "SelectedVoice" : Form1.synth1.SetSelectedProgramIndex(Val_Int(l)) : If Not LoadAll Then Form1.ListBox_Programs.SelectedIndex = Val_Int(l)
                    Case "ChordFileName" : Form1.ChordFileName = l
                    Case "SelectedTab" : Form1.tabControl1.SelectedIndex = Val_Int(l)
                End Select
            Loop
            f.Close()
        End If
        LimitFormPosition(Form1)
    End Sub



    ' ==================================================================================================
    '  SAVE LOAD -- ListBox_Chords
    ' ==================================================================================================

    'Friend Sub Save_ChordFile(Optional ByVal filename As String = "Default")
    '    '
    '    Dim CompletePath As String = PlatformAdjustedFileName(Application.StartupPath & "\Media\Chords\" & filename & ".txt")
    '    Dim f As System.IO.StreamWriter
    '    f = IO.File.CreateText(CompletePath)
    '    '
    '    For Each listitem As String In Form1.ListBox_Chords.Items
    '        f.WriteLine(listitem)
    '    Next
    '    '
    '    f.Close()
    'End Sub

    Friend Sub Load_ChordFile(Optional ByVal filename As String = "Default")
        Dim l As String
        Dim CompletePath As String = PlatformAdjustedFileName(Application.StartupPath & "\Media\Chords\" & filename & ".txt")
        If My.Computer.FileSystem.FileExists(CompletePath) Then
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(CompletePath)

            Form1.ListBox_Chords.Items.Clear()

            Do While Not f.EndOfStream
                l = f.ReadLine().Trim
                If l <> "" Then
                    If Not l.StartsWith("/") Then
                        Form1.ListBox_Chords.Items.Add(l)
                    End If
                End If
            Loop
            f.Close()
        End If
        If Form1.ListBox_Chords.Items.Count > 0 Then
            Form1.ListBox_Chords.SelectedIndex = 0
        End If
    End Sub


    ' ==================================================================================================
    '  SAVE IMAGE
    ' ==================================================================================================
    Public Sub SaveImage(ByVal img As Image, _
                         ByVal filename As String, _
                         ByVal extension As String, _
                         ByVal Quality As Int32)

        extension = LCase(extension)
        filename = RemoveExtension(filename)
        filename += "." & extension
        '
        If img Is Nothing Then Return
        Try
            File_Kill(filename)
            If extension = "jpg" Then
                Dim ImageEncoders() As ImageCodecInfo = ImageCodecInfo.GetImageEncoders()
                Dim myEncoder As System.Drawing.Imaging.Encoder = System.Drawing.Imaging.Encoder.Quality
                Dim myEncoderParameters As New EncoderParameters(1)
                Dim myEncoderParameter As New EncoderParameter(myEncoder, Quality)
                myEncoderParameters.Param(0) = myEncoderParameter
                img.Save(filename, ImageEncoders(1), myEncoderParameters)
            Else
                img.Save(filename, ImageFormatFromFileExtension(extension))
            End If
        Catch
            MsgBox("Image save error", MsgBoxStyle.Exclamation)
        End Try
    End Sub

    Private Function ImageFormatFromFileExtension(ByVal extension As String) As ImageFormat
        Select Case LCase(extension)
            Case "jpg" : Return ImageFormat.Jpeg
            Case "png" : Return ImageFormat.Png
            Case "tiff" : Return ImageFormat.Tiff
            Case "exif" : Return ImageFormat.Exif
            Case "emf" : Return ImageFormat.Emf
            Case "wmf" : Return ImageFormat.Wmf
            Case "gif" : Return ImageFormat.Gif
            Case "bmp" : Return ImageFormat.Bmp
                'Case "ico" : Return ImageFormat.Icon
            Case Else : Return ImageFormat.Jpeg
        End Select
    End Function

    Friend Sub File_Kill(ByVal filename As String)
        If My.Computer.FileSystem.FileExists(filename) Then
            My.Computer.FileSystem.DeleteFile(filename, FileIO.UIOption.OnlyErrorDialogs, FileIO.RecycleOption.DeletePermanently)
        End If
    End Sub

    Friend Sub SaveImage(ByVal Prefix As String, ByVal cnt As Control)
        Dim img As Image = GetImage(cnt)
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.Filter = "Portable network graphics (*.png)|*png|JPEG image (*.jpg)|*.jpg"
        sfd.FileName = Prefix & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            SaveImage(img, sfd.FileName, If(sfd.FilterIndex = 2, "jpg", "png"), 100)
        End If
    End Sub
    Friend Function GetImage(ByVal cnt As Control) As Bitmap
        Dim s As Size = cnt.Size
        Dim bmp As Bitmap = New Bitmap(s.Width, s.Height)
        cnt.DrawToBitmap(bmp, New Rectangle(0, 0, s.Width, s.Height))
        Return bmp
    End Function

End Module
