
Imports System.Collections.Generic
Imports System.Diagnostics
Imports LC.Synth

Public Class OscFMProgrammer
    Implements IProgrammable

	Public Enum ParameterId
		FMSource
	End Enum

	Public Enum FMSource
		None
		Lfo1
		Lfo2
		Envelope1
		Envelope2
	End Enum

    Private Const FMSourceCount As Integer = FMSource.Envelope2 + 1
	Private name As String = String.Empty
	Private osc As Oscillator
	Private emptyFMSource As EmptyMonoComponent
	Private env1 As AdsrEnvelope
	Private env2 As AdsrEnvelope
	Private lfo1 As Lfo
	Private lfo2 As Lfo
    Private _fmSource As FMSource = FMSource.None

    Public Sub New(ByVal name As String, _
                   ByVal osc As Oscillator, _
                   ByVal emptyFMSource As EmptyMonoComponent, _
                   ByVal env1 As AdsrEnvelope, _
                   ByVal env2 As AdsrEnvelope, _
                   ByVal lfo1 As Lfo, _
                   ByVal lfo2 As Lfo)

        If osc Is Nothing Then
            Throw New ArgumentNullException("osc")
        ElseIf emptyFMSource Is Nothing Then
            Throw New ArgumentNullException("emptyFMSource")
        ElseIf env1 Is Nothing Then
            Throw New ArgumentNullException("env1")
        ElseIf env2 Is Nothing Then
            Throw New ArgumentNullException("env2")
        ElseIf lfo1 Is Nothing Then
            Throw New ArgumentNullException("lfo1")
        ElseIf lfo2 Is Nothing Then
            Throw New ArgumentNullException("lfo2")
        End If

        Me.name = name
        Me.osc = osc
        Me.emptyFMSource = emptyFMSource
        Me.env1 = env1
        Me.env2 = env2
        Me.lfo1 = lfo1
        Me.lfo2 = lfo2
    End Sub



#Region "IProgrammable Members"

    Public Function GetParameterName(ByVal index As Integer) As String Implements IProgrammable.GetParameterName
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        '
        Dim result As String = ""
        Dim s As String = name
        If Not String.IsNullOrEmpty(s) Then s = s & " "
        '
        Select Case CType(index, ParameterId)
            Case ParameterId.FMSource
                result = s & "FM Source"
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
        Return result
    End Function

    Public Function GetParameterLabel(ByVal index As Integer) As String Implements IProgrammable.GetParameterLabel
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        Dim result As String = ""
        Select Case CType(index, ParameterId)
            Case ParameterId.FMSource
                result = "Source"
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
        Return result
    End Function

    Public Function GetParameterDisplay(ByVal index As Integer) As String Implements IProgrammable.GetParameterDisplay
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        Dim result As String = ""
        Select Case CType(index, ParameterId)
            Case ParameterId.FMSource
                result = _fmSource.ToString()
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
        Return result
    End Function

    Public Function GetParameterValue(ByVal index As Integer) As Single Implements IProgrammable.GetParameterValue
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        End If
        Dim result As Single = 0
        Select Case CType(index, ParameterId)
            Case ParameterId.FMSource
                result = CSng(_fmSource / (FMSourceCount - 1))
            Case Else
                Debug.Fail("Unhandled parameter")
        End Select
        Return result
    End Function

    Public Sub SetParameterValue(ByVal index As Integer, ByVal value As Single) Implements IProgrammable.SetParameterValue
        If index < 0 OrElse index >= ParameterCount Then
            Throw New ArgumentOutOfRangeException("index")
        ElseIf value < 0 OrElse value > 1 Then
            Throw New ArgumentOutOfRangeException("value")
        End If
        '
        Select Case CType(index, ParameterId)
            Case ParameterId.FMSource
                _fmSource = CType(Math.Round(value * (FMSourceCount - 1)), FMSource)

                Select Case _fmSource
                    Case FMSource.None
                        osc.FMModulator = emptyFMSource

                    Case FMSource.Envelope1
                        osc.FMModulator = env1

                    Case FMSource.Envelope2
                        osc.FMModulator = env2
                        
                    Case FMSource.Lfo1
                        osc.FMModulator = lfo1
                        
                    Case FMSource.Lfo2
                        osc.FMModulator = lfo2

                    Case Else
                        Debug.Fail("Unhandled FM Source")

                End Select

            Case Else
                Debug.Fail("Unhandled parameter")

        End Select
    End Sub

    Public ReadOnly Property ParameterCount() As Integer Implements IProgrammable.ParameterCount
        Get
            Return ParameterId.FMSource + 1
        End Get
    End Property

#End Region

End Class
